local Menu = require("Menu")
--------------------[-----------------------------[ Localising / Variables / Utilisation ]-----------------------------]--------------------
local Snake
local menu_font
local directionQueue
local foodPosition

local Game = {}

--- [ Map Setup/Math ] ---

--local Menu.gridXCount --= 40 -- Menu.gridXCount is equal to A*30, Width is equal to X
--local Menu.Menu.gridYCount --= 40 -- Menu.gridYCount is equal to A*30, Height is equal to Y

--- [ Timer ] ---
local timer = 0

--- [ Debug ] ---
local Debug = 0
-----------------------------[-----------------------------[ Main ]-----------------------------]-----------------------------

local function moveFood()
    local possibleFoodPositions = {}

    for foodX = 1, Menu.gridXCount do
        for foodY = 1, Menu.gridYCount do
            local possible = true

            for segmentIndex, segment in ipairs(Snake.Segments) do
                if foodX == segment.x and foodY == segment.y then
                    possible = false
                end
            end

            if possible then
                table.insert(possibleFoodPositions, {x = foodX, y = foodY})
            end
        end
    end

    foodPosition = possibleFoodPositions[love.math.random(#possibleFoodPositions)]
end


---[-[-[-[-[-[-[-[*** [ love.load() ] ***]-]-]-]-]-]-]-]---

function Game:load()
    menu_font = love.graphics.newFont('PixelFJVerdana12pt_0.ttf', 20)
    love.graphics.setFont(menu_font)

    directionQueue = {'right'}

    Snake = {x = 100,
             y = 500,
             w = 20,
             h = 20,
             direction = "right",
             speed = 200,
             cellSize = 15,
             Alive = true,
             R = 0,
             G = 255,
             B = 0,
             Segments = {
             {x = 3, y = 1},
             {x = 2, y = 1},
             {x = 1, y = 1}
             }
             }

    foodPosition = {
        x = love.math.random(1, Menu.gridXCount),
        y = love.math.random(1, Menu.gridYCount)
    }

    moveFood()

    love.window.setMode(Menu.gridXCount * Snake.cellSize, Menu.gridYCount * Snake.cellSize)

    if Menu.Active == 1 then
        Menu:load()
    end
end

---[-[-[-[-[-[-[-[*** [ love.update(dt) ] ***]-]-]-]-]-]-]-]---

function Game:update(dt)
    if Menu.Active == 1 then
        Menu:update()
    else
        --- [ Snake Updating ] ---

        timer = timer + dt
        if Snake.Alive == true then
            local timerLimit = 0.15
            if timer >= timerLimit then
                timer = timer - timerLimit

                local nextXPosition = Snake.Segments[1].x
                local nextYPosition = Snake.Segments[1].y

                --- [ Cleanup ] ---
                if #directionQueue > 1 then
                    table.remove(directionQueue, 1)
                end

                --- [ Key handling ] ---
                if directionQueue[1] == 'right' then
                    nextXPosition = nextXPosition + 1
                    if nextXPosition > Menu.gridXCount then
                        nextXPosition = 1
                    end
                elseif directionQueue[1] == 'left' then
                    nextXPosition = nextXPosition - 1
                    if nextXPosition < 1 then
                        nextXPosition = Menu.gridXCount
                    end
                elseif directionQueue[1] == 'down' then
                    nextYPosition = nextYPosition + 1
                    if nextYPosition > Menu.gridYCount then
                        nextYPosition = 1
                    end
                elseif directionQueue[1] == 'up' then
                    nextYPosition = nextYPosition - 1
                    if nextYPosition < 1 then
                        nextYPosition = Menu.gridYCount
                    end
                end

                --- [ Collision check ] ---
                for segmentIndex, segment in ipairs(Snake.Segments) do
                    if segmentIndex ~= #Snake.Segments
                    and nextXPosition == segment.x
                    and nextYPosition == segment.y then
                        Snake.Alive = false
                    end
                end

                --- [ Tail Manager ] ---
                table.insert(Snake.Segments, 1, {x = nextXPosition, y = nextYPosition})
                table.remove(Snake.Segments)
                table.insert(Snake.Segments, 1, {x = nextXPosition, y = nextYPosition})

                if Snake.Segments[1].x == foodPosition.x and Snake.Segments[1].y == foodPosition.y then
                    table.insert(Snake.Segments, 1, {x = nextXPosition, y = nextYPosition})
                    moveFood()
                else
                    table.remove(Snake.Segments)
                end
            end
            --- [ Reset/ Game over ] ---
        elseif timer >= 2 then
            Menu.Active = 1
            love.load()
        end

    end
end

---[-[-[-[-[-[-[-[*** [ love.draw() ] ***]-]-]-]-]-]-]-]---

function Game:draw()
    if Menu.Active == 1 then
        Menu:draw()
    else

        --- [ Bounding Box ] ---
        love.graphics.setColor(255, .28, .28)
        love.graphics.rectangle(
                'line',
                0,
                0,
                Menu.gridXCount * Snake.cellSize,
                Menu.gridYCount * Snake.cellSize
        )

        --- [ Cell function ] ---
        local function drawCell(x, y)
            love.graphics.rectangle(
                    'fill',
                    (x - 1) * Snake.cellSize,
                    (y - 1) * Snake.cellSize,
                    Snake.cellSize - 1,
                    Snake.cellSize - 1
            )
        end

        --- [ Snake color ] ---
        for segmentIndex, segment in ipairs(Snake.Segments) do
            if Snake.Alive then
                love.graphics.setColor(Snake.R, Snake.G, Snake.B)
            else
                love.graphics.setColor(.5, .5, .5)
            end
            drawCell(segment.x, segment.y)
        end

        love.graphics.setColor(1, .3, .3)
        drawCell(foodPosition.x, foodPosition.y)


        --- [ Debug ] ---
        if Debug == 1 then
        for directionIndex, direction in ipairs(directionQueue) do
            love.graphics.setColor(1, 1, 1)
            love.graphics.print('directionQueue['..directionIndex..']: '..direction, 15, 15 * directionIndex)
        end
        else
            love.graphics.print("", 15, 15)
        end
    end
end

---[-[-[-[-[-[-[-[*** [ love.keypressed(key) ] ***]-]-]-]-]-]-]-]---

function Game:keypressed(key)
    if Menu.Active == 1 then
        Menu.keypressed(key)
    else

        --- [ Key presses ] ---

        if key == 'right'
                and directionQueue[#directionQueue] ~= 'right'
                and directionQueue[#directionQueue] ~= 'left' then
            table.insert(directionQueue, 'right')

        elseif key == 'left'
                and directionQueue[#directionQueue] ~= 'left'
                and directionQueue[#directionQueue] ~= 'right' then
            table.insert(directionQueue, 'left')

        elseif key == 'up'
                and directionQueue[#directionQueue] ~= 'up'
                and directionQueue[#directionQueue] ~= 'down' then
            table.insert(directionQueue, 'up')

        elseif key == 'down'
                and directionQueue[#directionQueue] ~= 'down'
                and directionQueue[#directionQueue] ~= 'up' then
            table.insert(directionQueue, 'down')

        elseif key == '`' then
            Debug = Debug + 1
            if Debug == 2 then
                Debug = 0
            end
        end
    end
end

return Game, Snake
















